/**************************************************************/
/* Podprogramy pro obsluhu pameti EEprom AT24LC256            */               
/* LAZNICKA Pavel                                             */
/* Posledni uprava dne: 08.01.2005                            */ 
/**************************************************************/

#include <8052.h>
#include <stdlib.h>

/* Pripojeni pameti k systemu                                 */
/* Program je urcen pro vyvojovou desticku, ktera obsahuje    */
/* procesor P89C51RC+JN, seriovou I2C EEprom 24LC256 a        */            
/* seriovy interface ICL232 pro komunikaci                    */      
/* Pri navrhu jednostranneho plosneho spoje doslo k prohozeni */ 
/* privodu k seriove pameti EEprom typu 24C256                */
/* BASTL 	- univezalni plosny spoj rucne vydratovany        */
/* TISTAK 	- uhledna desticka jednostranneho plosneho spoje  */

#define TISTAK

/* Pripojeni vyvodu k univerzalni rucne vydratovane desce     */
/* BASTL 	- univezalni plosny spoj rucne vydratovany        */

#ifdef  BASTL
#define Eeprom_SDA P20      /* Port P2   Bit 00               */
#define Eeprom_SCL P21      /* Port P2   Bit 01               */  
#define Eeprom_WP  P22      /* Port P2   Bit 02               */
#endif

/* Pripojeni vyvodu na DPS cislo: 135-53-999 provedeni 01     */
/* TISTAK - uhledna desticka jednostranneho plosneho spoje    */

#ifdef  TISTAK
#define Eeprom_WP  P20      /* Port P2   Bit 00               */ 
#define Eeprom_SCL P21      /* Port P2   Bit 01               */
#define Eeprom_SDA P22      /* Port P2   Bit 02               */  
#endif

/* Komunikacni protokol - Prikazy pro praci   s  EEprom       */

/* @RAAAAXX + ENTER 	- Cteni jednoho bajtu z  EEprom       */
/* @WAAAADD + ENTER		- Zapis jednoho bajtu do EEprom       */

/* Deklarace konstant pouzitych v programu                    */

#define Doba_EEP    0x01     /* 1 ... doba zpozdeni EEPROM    */

/* Deklarace promennych pouzitych v programu                  */

unsigned char Error;

/* Deklarace funkci pouzitych v programu                      */

void Eeprom_Delay(unsigned short int Time);
void Eeprom_Start(void);
void Eeprom_Stop(void);
void Eeprom_Clock(void);
void Eeprom_Send_Byte(unsigned char Value);
unsigned char Eeprom_Read_Osum_Bitu(void);
void Eeprom_ACK_Test(void);

/* EEPROM_DELAY Funkce generuje casove zpozdeni TIME          */
/* Maximalni hodnota parametru Time je 65535 tj. cca 0.65535s */  

void Eeprom_Delay(unsigned short int Time)
{       
	while(( --Time ) != 0 );  
}

/* EEPROM_INIT generuje pocatecni nastaveni portu pro EEprom  */

void Eeprom_Init(void)
{
	Eeprom_SDA = 1;
	Eeprom_SCL = 1;
}

/* EEPROM_START funkce generuje START podminku pro I2C        */

void Eeprom_Start(void)
{
	Eeprom_SDA = 1;
	Eeprom_SCL = 1;
    Eeprom_Delay( Doba_EEP );
	Eeprom_SDA = 0;	
    Eeprom_Delay( Doba_EEP );
	Eeprom_SCL = 0;
    Eeprom_Delay( Doba_EEP );
}	 

/* EEPROM_STOP funkce generuje STOP podminku pro I2C          */

void Eeprom_Stop(void)
{
	Eeprom_SDA = 0;	
    Eeprom_Delay( Doba_EEP );
	Eeprom_SCL = 1;
    Eeprom_Delay( Doba_EEP );
    Eeprom_SDA = 1;
    Eeprom_Delay( Doba_EEP );
}

/* EEPROM_CLOCK funkce vyvola jeden seriovy hodinovy impuls   */

void Eeprom_Clock(void)
{
    Eeprom_SCL = 0;    							/* Clock Low  */
    Eeprom_Delay( Doba_EEP );
    Eeprom_SCL = 1;    							/* Clock High */
    Eeprom_Delay( Doba_EEP );
    Eeprom_SCL = 0;    							/* Clock Low  */
    Eeprom_Delay( Doba_EEP );
}

/* EEPROM_SEND_BYTE funkce posle jeden bajt do pameti EEprom  */

void Eeprom_Send_Byte(unsigned char Value)
{
 	unsigned char Index;
	
	for ( Index = 0x80 ; Index > 0 ; Index /= 2 )
 	{ 	
	 	if (Index & Value) Eeprom_SDA=1;
  		else Eeprom_SDA=0;
  		Eeprom_Delay(Doba_EEP);
  		Eeprom_SCL=1;      					 /* Clock for I2C */
  		Eeprom_Delay(Doba_EEP);
  		Eeprom_SCL=0;
 	}
}

unsigned char Eeprom_Read_Osum_Bitu(void)
{
	unsigned char Value;
	unsigned char Index;
		
	Value = 0x00;
	
    for ( Index = 0x80 ; Index > 0 ; Index /= 2 )
    {
     	Eeprom_SCL = 1;        
    	Eeprom_Delay( Doba_EEP );
    	if ( Eeprom_SDA == 1 ) { Value = Value | Index; }
    	Eeprom_Delay( Doba_EEP );  
      	Eeprom_SCL = 0; 
    	Eeprom_Delay( Doba_EEP );  
	}
	
	return Value;
}	
	
/* EEPROM_ACK_TEST Testuje hodnotu ACK bitu z pameti EEprom   */

void Eeprom_ACK_Test(void)
{
    Eeprom_Delay( Doba_EEP );
	Eeprom_SDA = 1;        /* uvolneni vodice SDA             */
	Eeprom_Delay( Doba_EEP );
    Eeprom_SCL = 1;
	Eeprom_Delay( Doba_EEP );    
   	if (Eeprom_SDA == 1) { Error = 1; }  /* test ACK          */
	Eeprom_Delay( Doba_EEP );
    Eeprom_SCL = 0;    
    Eeprom_Delay( Doba_EEP );
}
        
/* EEPROM_READ_BYTE precte jeden bajt z pameti Eeprom         */

unsigned char Eeprom_Read_Byte(unsigned short int EE_Adr)
{
    unsigned char EE_Data;
    unsigned char EE_Adr_Low;
    unsigned char EE_Adr_Hig;
        
/*  Nejdrive je nutno vygenerovat START podminku              */
	
    Eeprom_Start();	

/*  Potom je nutno poslat Device Address Byte - WRITE         */    	
    
	Eeprom_Send_Byte( 0xA0);
	        
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();
	
/*  Potom je nutno poslat vyssi bajt adresy                   */    	
    	
	EE_Adr_Hig = ( EE_Adr & 0xFF00 ) / 256;
	Eeprom_Send_Byte( EE_Adr_Hig );
    	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat nizsi bajt adresy                   */    	
    
	EE_Adr_Low = ( EE_Adr & 0x00FF );
	Eeprom_Send_Byte( EE_Adr_Low );	
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();		

/*  Potom je nutno vygenerovat STOP podminku                  */       

	Eeprom_Stop();  	
		
/*  Pro jistotu jeste chvili pockam                           */

	Eeprom_Delay( Doba_EEP );	
		
/*  Potom je nutno vygenerovat druhou START podminku          */
	
    Eeprom_Start();		
	
/*  Potom je nutno poslat Device Address Byte - READ          */    	
    	
	Eeprom_Send_Byte( 0xA1 );
		
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();			

/*  Potom musim uvolnit datovou sbernici, aby za ni           */	
/*  mohla zacit tahat ta pamet                                */

	Eeprom_SDA = 1;
		
/*  Ted by mi pamet mela poslat 8 bitu prectenych dat         */	
/*  Cyklus pro 8 bitu prectenych dat z pameti EEprom          */	

	EE_Data = Eeprom_Read_Osum_Bitu();

/*  Nyni jiz nebudu vysilat potvrzovaci ACK bit               */
/*  NO ACK,  SDA = 1                                          */
/*  a vygeneruji devaty hodinovy  impulz                      */

 	Eeprom_SDA = 1;
 	Eeprom_Clock();
	         
/*  Nakonec je nutno vygenerovat STOP podminku                */       

	Eeprom_Stop();  

	return EE_Data;			
} 

/* EEPROM_READ_WORD precte dva bajty z pameti Eeprom         */

unsigned short int Eeprom_Read_Word(unsigned short int EE_Adr)
{
    unsigned char EE_Data_Low;
    unsigned char EE_Data_Hig;
    unsigned char EE_Adr_Low;
    unsigned char EE_Adr_Hig;
        
/*  Nejdrive je nutno vygenerovat START podminku              */
	
    Eeprom_Start();	

/*  Potom je nutno poslat Device Address Byte - WRITE         */    	
    
	Eeprom_Send_Byte( 0xA0);
	        
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();
	
/*  Potom je nutno poslat vyssi bajt adresy                   */    	
    	
	EE_Adr_Hig = ( EE_Adr & 0xFF00 ) / 256;
	Eeprom_Send_Byte( EE_Adr_Hig );
    	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat nizsi bajt adresy                   */    	
    
	EE_Adr_Low = ( EE_Adr & 0x00FF );
	Eeprom_Send_Byte( EE_Adr_Low );	
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();		

/*  Potom je nutno vygenerovat STOP podminku                  */       

	Eeprom_Stop();  	
		
/*  Pro jistotu jeste chvili pockam                           */

	Eeprom_Delay( Doba_EEP );	
		
/*  Potom je nutno vygenerovat druhou START podminku          */
	
    Eeprom_Start();		
	
/*  Potom je nutno poslat Device Address Byte - READ          */    	
    	
	Eeprom_Send_Byte( 0xA1 );
		
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();			

/*  Potom musim uvolnit datovou sbernici, aby za ni           */	
/*  mohla zacit tahat ta pamet                                */

	Eeprom_SDA = 1;
		
/*  Ted by mi pamet mela poslat 8 bitu prectenych dat         */	
/*  Cyklus pro 8 bitu prectenych dat z pameti EEprom          */	

	EE_Data_Low = Eeprom_Read_Osum_Bitu();

/*  Procesor porvrdi prijem pomoci ACK bitu                   */
/*  a vygeneruji devaty hodinovy  impulz                      */

 	Eeprom_SDA = 0;
 	Eeprom_Clock();		

/*  Potom musim uvolnit datovou sbernici, aby za ni           */	
/*  mohla zacit tahat ta pamet                                */

	Eeprom_SDA = 1; 		
	
/*  Ted by mi pamet mela poslat 8 bitu prectenych dat         */	
/*  Cyklus pro 8 bitu prectenych dat z pameti EEprom          */	

	EE_Data_Hig = Eeprom_Read_Osum_Bitu();	
	
/*  Nyni jiz nebudu vysilat potvrzovaci ACK bit               */
/*  NO ACK,  SDA = 1                                          */
/*  a vygeneruji devaty hodinovy  impulz                      */

 	Eeprom_SDA = 1;
 	Eeprom_Clock();	
		         
/*  Nakonec je nutno vygenerovat STOP podminku                */       

	Eeprom_Stop();  

	return (( 256 * EE_Data_Hig ) + EE_Data_Low );			
} 

/* EEPROM_WRITE_BYTE - zapise jeden bajt do pameti Eeprom    */

unsigned char Eeprom_Write_Byte
(unsigned short int EE_Adr,unsigned char EE_Dat)
{
	unsigned char EE_Adr_Low;
    unsigned char EE_Adr_Hig;
		
/*  Nejdrive je nutno vygenerovat START podminku              */
	
    Eeprom_Start();	

/*  Potom je nutno poslat Device Address Byte - WRITE         */    	
        
	Eeprom_Send_Byte( 0xA0);	
		
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();
    
/*  Potom je nutno poslat vyssi bajt adresy                   */    	
    
	EE_Adr_Hig = ( EE_Adr & 0xFF00 ) / 256;
	Eeprom_Send_Byte( EE_Adr_Hig );	
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat nizsi bajt adresy                   */    	
    	
	EE_Adr_Low = ( EE_Adr & 0x00FF );
	Eeprom_Send_Byte( EE_Adr_Low );	
			
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat bajt dat urcenych pro zapis         */  	

	Eeprom_Send_Byte( EE_Dat );		
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	

/*  Nakonec je nutno vygenerovat STOP podminku                */	
				
	Eeprom_Stop();        

/*  Funkce vrati vysledek testovani ACK bitu                  */	
	
	return Error;
} 

/* EEPROM_WRITE_WORD - zapise jeden bajt do pameti Eeprom    */

unsigned char Eeprom_Write_Word
(unsigned short int EE_Adr,unsigned char EE_Dat)
{
	unsigned char EE_Adr_Low;
    unsigned char EE_Adr_Hig;
    unsigned char EE_Dat_Low;
    unsigned char EE_Dat_Hig;
		
/*  Nejdrive je nutno vygenerovat START podminku              */
	
    Eeprom_Start();	

/*  Potom je nutno poslat Device Address Byte - WRITE         */    	
        
	Eeprom_Send_Byte( 0xA0);	
		
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();
    
/*  Potom je nutno poslat vyssi bajt adresy                   */    	
    
	EE_Adr_Hig = ( EE_Adr & 0xFF00 ) / 256;
	Eeprom_Send_Byte( EE_Adr_Hig );	
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat nizsi bajt adresy                   */    	
    	
	EE_Adr_Low = ( EE_Adr & 0x00FF );
	Eeprom_Send_Byte( EE_Adr_Low );	
			
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	
		
/*  Potom je nutno poslat nizsi bajt dat urcenych pro zapis   */  	

	EE_Dat_Low = ( EE_Dat & 0x00FF );
	Eeprom_Send_Byte( EE_Dat_Low );		
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();	

/*  Potom je nutno poslat vyssi bajt dat urcenych pro zapis   */  	

	EE_Dat_Hig = ( EE_Dat & 0xFF00 ) / 256;
	Eeprom_Send_Byte( EE_Dat_Hig );		
	
/*  Potom testuji hodnotu ACK bitu prijateho z pameti         */

	Eeprom_ACK_Test();		
	
/*  Nakonec je nutno vygenerovat STOP podminku                */	
				
	Eeprom_Stop();        

/*  Funkce vrati vysledek testovani ACK bitu                  */	
	
	return Error;
} 

